@@ -11,9 +11,10 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+from __future__ import annotations
 
+from typing import TYPE_CHECKING, Any, Tuple, Type, Union
 from uuid import UUID
-from warnings import warn
 
 """Tools for representing BSON binary data.
 """
@@ -57,6 +58,11 @@ by :mod:`bson` using this subtype when using
 """
 
 
+if TYPE_CHECKING:
+    from array import array as _array
+    from mmap import mmap as _mmap
+
+
 class UuidRepresentation:
     UNSPECIFIED = 0
     """An unspecified UUID representation.
@@ -69,7 +75,7 @@ class UuidRepresentation:
     code. When decoding a BSON binary field with a UUID subtype, a
     :class:`~bson.binary.Binary` instance will be returned instead of a
     :class:`uuid.UUID` instance.
-    
+
     See :ref:`unspecified-representation-details` for details.
 
     .. versionadded:: 3.11
@@ -81,7 +87,7 @@ class UuidRepresentation:
     :class:`uuid.UUID` instances will automatically be encoded to
     and decoded from BSON binary, using RFC-4122 byte order with
     binary subtype :data:`UUID_SUBTYPE`.
-    
+
     See :ref:`standard-representation-details` for details.
 
     .. versionadded:: 3.11
@@ -93,7 +99,7 @@ class UuidRepresentation:
     :class:`uuid.UUID` instances will automatically be encoded to
     and decoded from BSON binary, using RFC-4122 byte order with
     binary subtype :data:`OLD_UUID_SUBTYPE`.
-    
+
     See :ref:`python-legacy-representation-details` for details.
 
     .. versionadded:: 3.11
@@ -105,7 +111,7 @@ class UuidRepresentation:
     :class:`uuid.UUID` instances will automatically be encoded to
     and decoded from BSON binary subtype :data:`OLD_UUID_SUBTYPE`,
     using the Java driver's legacy byte order.
-    
+
     See :ref:`java-legacy-representation-details` for details.
 
     .. versionadded:: 3.11
@@ -117,7 +123,7 @@ class UuidRepresentation:
     :class:`uuid.UUID` instances will automatically be encoded to
     and decoded from BSON binary subtype :data:`OLD_UUID_SUBTYPE`,
     using the C# driver's legacy byte order.
-    
+
     See :ref:`csharp-legacy-representation-details` for details.
 
     .. versionadded:: 3.11
@@ -153,17 +159,20 @@ CSHARP_LEGACY = UuidRepresentation.CSHARP_LEGACY
 """
 
 ALL_UUID_SUBTYPES = (OLD_UUID_SUBTYPE, UUID_SUBTYPE)
-ALL_UUID_REPRESENTATIONS = (UuidRepresentation.UNSPECIFIED,
-                            UuidRepresentation.STANDARD,
-                            UuidRepresentation.PYTHON_LEGACY,
-                            UuidRepresentation.JAVA_LEGACY,
-                            UuidRepresentation.CSHARP_LEGACY)
+ALL_UUID_REPRESENTATIONS = (
+    UuidRepresentation.UNSPECIFIED,
+    UuidRepresentation.STANDARD,
+    UuidRepresentation.PYTHON_LEGACY,
+    UuidRepresentation.JAVA_LEGACY,
+    UuidRepresentation.CSHARP_LEGACY,
+)
 UUID_REPRESENTATION_NAMES = {
-    UuidRepresentation.UNSPECIFIED: 'UuidRepresentation.UNSPECIFIED',
-    UuidRepresentation.STANDARD: 'UuidRepresentation.STANDARD',
-    UuidRepresentation.PYTHON_LEGACY: 'UuidRepresentation.PYTHON_LEGACY',
-    UuidRepresentation.JAVA_LEGACY: 'UuidRepresentation.JAVA_LEGACY',
-    UuidRepresentation.CSHARP_LEGACY: 'UuidRepresentation.CSHARP_LEGACY'}
+    UuidRepresentation.UNSPECIFIED: "UuidRepresentation.UNSPECIFIED",
+    UuidRepresentation.STANDARD: "UuidRepresentation.STANDARD",
+    UuidRepresentation.PYTHON_LEGACY: "UuidRepresentation.PYTHON_LEGACY",
+    UuidRepresentation.JAVA_LEGACY: "UuidRepresentation.JAVA_LEGACY",
+    UuidRepresentation.CSHARP_LEGACY: "UuidRepresentation.CSHARP_LEGACY",
+}
 
 MD5_SUBTYPE = 5
 """BSON binary subtype for an MD5 hash.
@@ -175,6 +184,13 @@ COLUMN_SUBTYPE = 7
 .. versionadded:: 4.0
 """
 
+SENSITIVE_SUBTYPE = 8
+"""BSON binary subtype for sensitive data.
+
+.. versionadded:: 4.5
+"""
+
+
 USER_DEFINED_SUBTYPE = 128
 """BSON binary subtype for any user defined structure.
 """
@@ -189,18 +205,16 @@ class Binary(bytes):
     what should be considered a string when we encode to BSON.
 
     Raises TypeError if `data` is not an instance of :class:`bytes`
-    (:class:`str` in python 2) or `subtype` is not an instance of
-    :class:`int`. Raises ValueError if `subtype` is not in [0, 256).
+    or `subtype` is not an instance of :class:`int`.
+    Raises ValueError if `subtype` is not in [0, 256).
 
     .. note::
-      In python 3 instances of Binary with subtype 0 will be decoded
-      directly to :class:`bytes`.
+      Instances of Binary with subtype 0 will be decoded directly to :class:`bytes`.
 
-    :Parameters:
-      - `data`: the binary data to represent. Can be any bytes-like type
+    :param data: the binary data to represent. Can be any bytes-like type
         that implements the buffer protocol.
-      - `subtype` (optional): the `binary subtype
-        <http://bsonspec.org/#/specification>`_
+    :param subtype: the `binary subtype
+        <https://bsonspec.org/spec.html>`_
         to use
 
     .. versionchanged:: 3.9
@@ -208,8 +222,13 @@ class Binary(bytes):
     """
 
     _type_marker = 5
+    __subtype: int
 
-    def __new__(cls, data, subtype=BINARY_SUBTYPE):
+    def __new__(
+        cls: Type[Binary],
+        data: Union[memoryview, bytes, _mmap, _array[Any]],
+        subtype: int = BINARY_SUBTYPE,
+    ) -> Binary:
         if not isinstance(subtype, int):
             raise TypeError("subtype must be an instance of int")
         if subtype >= 256 or subtype < 0:
@@ -220,7 +239,9 @@ class Binary(bytes):
         return self
 
     @classmethod
-    def from_uuid(cls, uuid, uuid_representation=UuidRepresentation.STANDARD):
+    def from_uuid(
+        cls: Type[Binary], uuid: UUID, uuid_representation: int = UuidRepresentation.STANDARD
+    ) -> Binary:
         """Create a BSON Binary object from a Python UUID.
 
         Creates a :class:`~bson.binary.Binary` object from a
@@ -231,9 +252,8 @@ class Binary(bytes):
         Raises :exc:`TypeError` if `uuid` is not an instance of
         :class:`~uuid.UUID`.
 
-        :Parameters:
-          - `uuid`: A :class:`uuid.UUID` instance.
-          - `uuid_representation`: A member of
+        :param uuid: A :class:`uuid.UUID` instance.
+        :param uuid_representation: A member of
             :class:`~bson.binary.UuidRepresentation`. Default:
             :const:`~bson.binary.UuidRepresentation.STANDARD`.
             See :ref:`handling-uuid-data-example` for details.
@@ -244,8 +264,9 @@ class Binary(bytes):
             raise TypeError("uuid must be an instance of uuid.UUID")
 
         if uuid_representation not in ALL_UUID_REPRESENTATIONS:
-            raise ValueError("uuid_representation must be a value "
-                             "from bson.binary.UuidRepresentation")
+            raise ValueError(
+                "uuid_representation must be a value from bson.binary.UuidRepresentation"
+            )
 
         if uuid_representation == UuidRepresentation.UNSPECIFIED:
             raise ValueError(
@@ -254,7 +275,8 @@ class Binary(bytes):
                 "converted to bson.Binary instances using "
                 "bson.Binary.from_uuid() or a different UuidRepresentation "
                 "can be configured. See the documentation for "
-                "UuidRepresentation for more information.")
+                "UuidRepresentation for more information."
+            )
 
         subtype = OLD_UUID_SUBTYPE
         if uuid_representation == UuidRepresentation.PYTHON_LEGACY:
@@ -271,7 +293,7 @@ class Binary(bytes):
 
         return cls(payload, subtype)
 
-    def as_uuid(self, uuid_representation=UuidRepresentation.STANDARD):
+    def as_uuid(self, uuid_representation: int = UuidRepresentation.STANDARD) -> UUID:
         """Create a Python UUID from this BSON Binary object.
 
         Decodes this binary object as a native :class:`uuid.UUID` instance
@@ -280,8 +302,7 @@ class Binary(bytes):
         Raises :exc:`ValueError` if this :class:`~bson.binary.Binary` instance
         does not contain a UUID.
 
-        :Parameters:
-          - `uuid_representation`: A member of
+        :param uuid_representation: A member of
             :class:`~bson.binary.UuidRepresentation`. Default:
             :const:`~bson.binary.UuidRepresentation.STANDARD`.
             See :ref:`handling-uuid-data-example` for details.
@@ -289,12 +310,12 @@ class Binary(bytes):
         .. versionadded:: 3.11
         """
         if self.subtype not in ALL_UUID_SUBTYPES:
-            raise ValueError("cannot decode subtype %s as a uuid" % (
-                self.subtype,))
+            raise ValueError(f"cannot decode subtype {self.subtype} as a uuid")
 
         if uuid_representation not in ALL_UUID_REPRESENTATIONS:
-            raise ValueError("uuid_representation must be a value from "
-                             "bson.binary.UuidRepresentation")
+            raise ValueError(
+                "uuid_representation must be a value from bson.binary.UuidRepresentation"
+            )
 
         if uuid_representation == UuidRepresentation.UNSPECIFIED:
             raise ValueError("uuid_representation cannot be UNSPECIFIED")
@@ -312,36 +333,38 @@ class Binary(bytes):
             if self.subtype == UUID_SUBTYPE:
                 return UUID(bytes=self)
 
-        raise ValueError("cannot decode subtype %s to %s" % (
-                self.subtype, UUID_REPRESENTATION_NAMES[uuid_representation]))
+        raise ValueError(
+            f"cannot decode subtype {self.subtype} to {UUID_REPRESENTATION_NAMES[uuid_representation]}"
+        )
 
     @property
-    def subtype(self):
-        """Subtype of this binary data.
-        """
+    def subtype(self) -> int:
+        """Subtype of this binary data."""
         return self.__subtype
 
-    def __getnewargs__(self):
+    def __getnewargs__(self) -> Tuple[bytes, int]:  # type: ignore[override]
         # Work around http://bugs.python.org/issue7382
-        data = super(Binary, self).__getnewargs__()[0]
+        data = super().__getnewargs__()[0]
         if not isinstance(data, bytes):
-            data = data.encode('latin-1')
+            data = data.encode("latin-1")
         return data, self.__subtype
 
-    def __eq__(self, other):
+    def __eq__(self, other: Any) -> bool:
         if isinstance(other, Binary):
-            return ((self.__subtype, bytes(self)) ==
-                    (other.subtype, bytes(other)))
+            return (self.__subtype, bytes(self)) == (other.subtype, bytes(other))
         # We don't return NotImplemented here because if we did then
         # Binary("foo") == "foo" would return True, since Binary is a
         # subclass of str...
         return False
 
-    def __hash__(self):
-        return super(Binary, self).__hash__() ^ hash(self.__subtype)
+    def __hash__(self) -> int:
+        return super().__hash__() ^ hash(self.__subtype)
 
-    def __ne__(self, other):
+    def __ne__(self, other: Any) -> bool:
         return not self == other
 
-    def __repr__(self):
-        return "Binary(%s, %s)" % (bytes.__repr__(self), self.__subtype)
+    def __repr__(self) -> str:
+        if self.__subtype == SENSITIVE_SUBTYPE:
+            return f"<Binary(REDACTED, {self.__subtype})>"
+        else:
+            return f"Binary({bytes.__repr__(self)}, {self.__subtype})"
