@@ -13,15 +13,27 @@
 # permissions and limitations under the License.
 
 """Criteria to select some ServerDescriptions from a TopologyDescription."""
+from __future__ import annotations
+
+from typing import TYPE_CHECKING, Any, Mapping, Optional, Sequence, TypeVar, cast
 
 from pymongo.server_type import SERVER_TYPE
 
+if TYPE_CHECKING:
+    from pymongo.server_description import ServerDescription
+    from pymongo.topology_description import TopologyDescription
 
-class Selection(object):
+
+T = TypeVar("T")
+TagSet = Mapping[str, Any]
+TagSets = Sequence[TagSet]
+
+
+class Selection:
     """Input or output of a server selector function."""
 
     @classmethod
-    def from_topology_description(cls, topology_description):
+    def from_topology_description(cls, topology_description: TopologyDescription) -> Selection:
         known_servers = topology_description.known_servers
         primary = None
         for sd in known_servers:
@@ -29,86 +41,92 @@ class Selection(object):
                 primary = sd
                 break
 
-        return Selection(topology_description,
-                         topology_description.known_servers,
-                         topology_description.common_wire_version,
-                         primary)
+        return Selection(
+            topology_description,
+            topology_description.known_servers,
+            topology_description.common_wire_version,
+            primary,
+        )
 
-    def __init__(self,
-                 topology_description,
-                 server_descriptions,
-                 common_wire_version,
-                 primary):
+    def __init__(
+        self,
+        topology_description: TopologyDescription,
+        server_descriptions: list[ServerDescription],
+        common_wire_version: Optional[int],
+        primary: Optional[ServerDescription],
+    ):
         self.topology_description = topology_description
         self.server_descriptions = server_descriptions
         self.primary = primary
         self.common_wire_version = common_wire_version
 
-    def with_server_descriptions(self, server_descriptions):
-        return Selection(self.topology_description,
-                         server_descriptions,
-                         self.common_wire_version,
-                         self.primary)
+    def with_server_descriptions(self, server_descriptions: list[ServerDescription]) -> Selection:
+        return Selection(
+            self.topology_description, server_descriptions, self.common_wire_version, self.primary
+        )
 
-    def secondary_with_max_last_write_date(self):
+    def secondary_with_max_last_write_date(self) -> Optional[ServerDescription]:
         secondaries = secondary_server_selector(self)
         if secondaries.server_descriptions:
-            return max(secondaries.server_descriptions,
-                       key=lambda sd: sd.last_write_date)
+            return max(
+                secondaries.server_descriptions, key=lambda sd: cast(float, sd.last_write_date)
+            )
+        return None
 
     @property
-    def primary_selection(self):
+    def primary_selection(self) -> Selection:
         primaries = [self.primary] if self.primary else []
         return self.with_server_descriptions(primaries)
 
     @property
-    def heartbeat_frequency(self):
+    def heartbeat_frequency(self) -> int:
         return self.topology_description.heartbeat_frequency
 
     @property
-    def topology_type(self):
+    def topology_type(self) -> int:
         return self.topology_description.topology_type
 
-    def __bool__(self):
+    def __bool__(self) -> bool:
         return bool(self.server_descriptions)
 
-    def __getitem__(self, item):
+    def __getitem__(self, item: int) -> ServerDescription:
         return self.server_descriptions[item]
 
 
-def any_server_selector(selection):
+def any_server_selector(selection: T) -> T:
     return selection
 
 
-def readable_server_selector(selection):
+def readable_server_selector(selection: Selection) -> Selection:
     return selection.with_server_descriptions(
-        [s for s in selection.server_descriptions if s.is_readable])
+        [s for s in selection.server_descriptions if s.is_readable]
+    )
 
 
-def writable_server_selector(selection):
+def writable_server_selector(selection: Selection) -> Selection:
     return selection.with_server_descriptions(
-        [s for s in selection.server_descriptions if s.is_writable])
+        [s for s in selection.server_descriptions if s.is_writable]
+    )
 
 
-def secondary_server_selector(selection):
+def secondary_server_selector(selection: Selection) -> Selection:
     return selection.with_server_descriptions(
-        [s for s in selection.server_descriptions
-         if s.server_type == SERVER_TYPE.RSSecondary])
+        [s for s in selection.server_descriptions if s.server_type == SERVER_TYPE.RSSecondary]
+    )
 
 
-def arbiter_server_selector(selection):
+def arbiter_server_selector(selection: Selection) -> Selection:
     return selection.with_server_descriptions(
-        [s for s in selection.server_descriptions
-         if s.server_type == SERVER_TYPE.RSArbiter])
+        [s for s in selection.server_descriptions if s.server_type == SERVER_TYPE.RSArbiter]
+    )
 
 
-def writable_preferred_server_selector(selection):
+def writable_preferred_server_selector(selection: Selection) -> Selection:
     """Like PrimaryPreferred but doesn't use tags or latency."""
-    return (writable_server_selector(selection) or
-            secondary_server_selector(selection))
+    return writable_server_selector(selection) or secondary_server_selector(selection)
 
 
-def apply_single_tag_set(tag_set, selection):
+def apply_single_tag_set(tag_set: TagSet, selection: Selection) -> Selection:
     """All servers matching one tag set.
 
     A tag set is a dict. A server matches if its tags are a superset:
@@ -116,7 +134,8 @@ def apply_single_tag_set(tag_set, selection):
 
     The empty tag set {} matches any server.
     """
-    def tags_match(server_tags):
+
+    def tags_match(server_tags: Mapping[str, Any]) -> bool:
         for key, value in tag_set.items():
             if key not in server_tags or server_tags[key] != value:
                 return False
@@ -124,10 +143,11 @@ def apply_single_tag_set(tag_set, selection):
         return True
 
     return selection.with_server_descriptions(
-        [s for s in selection.server_descriptions if tags_match(s.tags)])
+        [s for s in selection.server_descriptions if tags_match(s.tags)]
+    )
 
 
-def apply_tag_sets(tag_sets, selection):
+def apply_tag_sets(tag_sets: TagSets, selection: Selection) -> Selection:
     """All servers match a list of tag sets.
 
     tag_sets is a list of dicts. The empty tag set {} matches any server,
@@ -144,11 +164,11 @@ def apply_tag_sets(tag_sets, selection):
     return selection.with_server_descriptions([])
 
 
-def secondary_with_tags_server_selector(tag_sets, selection):
+def secondary_with_tags_server_selector(tag_sets: TagSets, selection: Selection) -> Selection:
     """All near-enough secondaries matching the tag sets."""
     return apply_tag_sets(tag_sets, secondary_server_selector(selection))
 
 
-def member_with_tags_server_selector(tag_sets, selection):
+def member_with_tags_server_selector(tag_sets: TagSets, selection: Selection) -> Selection:
     """All near-enough members matching the tag sets."""
     return apply_tag_sets(tag_sets, readable_server_selector(selection))
